'use client';
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jsxRuntime = require('react/jsx-runtime');
var React = require('react');
var styled = require('styled-components');
var uiUtils = require('@bolttech/ui-utils');
var defaultTheme = require('@bolttech/default-theme');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var styled__default = /*#__PURE__*/_interopDefaultLegacy(styled);

const ScreenSizes = {
    LG: {
        min: 1440,
    },
    MD: {
        max: 1439,
        min: 768,
    },
    SM: {
        max: 767,
        min: 414,
    },
    XS: {
        max: 413,
    },
};
const MediaQueries = {
    LG: `(min-width: ${ScreenSizes.LG.min}px)`,
    MD: `(min-width: ${ScreenSizes.MD.min}px) and (max-width: ${ScreenSizes.MD.max}px)`,
    SM: `(min-width: ${ScreenSizes.SM.min}px) and (max-width: ${ScreenSizes.SM.max}px)`,
    XS: `(max-width: ${ScreenSizes.XS.max}px)`,
};

const PREFIX_REGEX = /([a-z]{2}:)/g;
/**
 * Generates CSS border styles based on utility class names found within the `className` parameter.
 * It utilizes regex patterns to identify border width and color classes, then maps these to corresponding
 * CSS rules based on the provided theme settings.
 *
 * Border width and color utility classes are expected to follow specific naming conventions, and
 * this function handles both responsive prefixes and different border sides.
 *
 * @param {CssType} param0 - An object containing the theme and className properties.
 * @returns {string} The generated CSS string for border styles.
 *
 * @example
 * // Assuming the theme has predefined border widths and colors
 * generateBorderStyles({ theme, className: 'bt-2 b-color-primary' });
 * // -> "border-top: 2px solid; border-color: #007bff;"
 */
const generateBorderStyles = ({ theme, className }) => {
    if (className && className.length > 0) {
        const BORDER_WIDTH_REGEX = /\b([a-z]{2}:)?\bb(t|b|l|r)?-([0-9]|(s|m|l|d|x|g)){1,3}/g;
        const BORDER_COLOR_REGEX = /\b([a-z]{2}:)?\bb(t|b|l|r)?-color-([A-z]|[0-9])+/g;
        const borderClasses = className.match(BORDER_WIDTH_REGEX);
        const borderColorClasses = className.match(BORDER_COLOR_REGEX);
        const propId = {
            t: 'border-top',
            b: 'border-bottom',
            l: 'border-left',
            r: 'border-right',
        };
        const colorPropId = {
            t: 'border-top-color',
            b: 'border-bottom-color',
            l: 'border-left-color',
            r: 'border-right-color',
        };
        const cssBorderWidth = borderClasses
            ?.map((classFound) => {
            const havePrefix = classFound.match(PREFIX_REGEX);
            const classNameWithoutPrefix = classFound.replace(PREFIX_REGEX, '');
            const borderSpecs = classNameWithoutPrefix.split('-');
            // Check if border variant exists
            if (borderSpecs[1].toUpperCase() in theme.spacing.borderWidth) {
                let rule = '';
                if (borderSpecs[0].length > 1) {
                    rule = `${propId[borderSpecs[0][1]]}: ${theme.spacing.borderWidth[borderSpecs[1].toUpperCase()]} solid;`;
                }
                else {
                    rule = `border: ${theme.spacing.borderWidth[borderSpecs[1].toUpperCase()]} solid;`;
                }
                if (havePrefix?.length) {
                    return `
              @media ${MediaQueries[havePrefix[0].replace(':', '').toUpperCase()]} {
                ${rule}
              }
            `;
                }
                return rule;
            }
            return '';
        })
            .join('');
        const cssBorderColor = borderColorClasses
            ?.map((classFound) => {
            const havePrefix = classFound.match(PREFIX_REGEX);
            const classNameWithoutPrefix = classFound.replace(PREFIX_REGEX, '');
            const borderSpecs = classNameWithoutPrefix.split('-');
            // Check if border variant exists
            if (borderSpecs[2] in theme.colors.border) {
                let rule = '';
                if (borderSpecs[0].length > 1) {
                    rule = `${colorPropId[borderSpecs[0][1]]}: ${theme.colors.border[borderSpecs[2]]};`;
                }
                else {
                    rule = `border-color: ${theme.colors.border[borderSpecs[2]]};`;
                }
                if (havePrefix?.length) {
                    return `
              @media ${MediaQueries[havePrefix[0].replace(':', '').toUpperCase()]} {
                ${rule}
              }
            `;
                }
                return rule;
            }
            return '';
        })
            .join('');
        return styled.css `
      ${cssBorderWidth}
      ${cssBorderColor}
    `;
    }
    return undefined;
};
/**
 * Generates CSS box-shadow styles based on utility class names found within the `className` parameter.
 * It utilizes a regex pattern to identify box-shadow level classes, then maps these to corresponding
 * CSS rules based on the provided theme settings.
 *
 * Box-shadow level utility classes are expected to follow a specific naming convention,
 * and this function also handles responsive prefixes.
 *
 * @param {CssType} param0 - An object containing the theme and className properties.
 * @returns {string} The generated CSS string for box-shadow styles.
 *
 * @example
 * // Assuming the theme has predefined box-shadow levels
 * generateBoxShadowStyles({ theme, className: 'bs-lvl2' });
 * // -> "box-shadow: 0px 4px 6px -1px rgba(0,0,0,0.1),0px 2px 4px -1px rgba(0,0,0,0.06);"
 */
const generateBoxShadowStyles = ({ theme, className }) => {
    const BOX_SHADOW_REGEX = /\b([a-z]{2}:)?\bbs-lvl[0-9]+/g;
    const boxShadowClassesFound = className?.match(BOX_SHADOW_REGEX);
    if (boxShadowClassesFound && boxShadowClassesFound.length > 0) {
        return boxShadowClassesFound.map((classFound) => {
            const havePrefix = classFound.match(PREFIX_REGEX);
            const shadowClassNameSplitted = classFound.split('-');
            const rule = `box-shadow: ${theme.effects.shadow[shadowClassNameSplitted[1]]};`;
            if (havePrefix?.length) {
                return `
          @media ${MediaQueries[havePrefix[0].replace(':', '').toUpperCase()]} {
            ${rule}
          }
        `;
            }
            return rule;
        });
    }
    return undefined;
};
const generateBorderRadiusStyles = ({ theme, className }) => {
    const BORDER_RADIUS_REGEX = /\b([a-z]{2}:)?\bb-radius-([A-z]|[0-9])+/g;
    const borderRadiusClassesFound = className?.match(BORDER_RADIUS_REGEX);
    if (borderRadiusClassesFound && borderRadiusClassesFound.length > 0) {
        return borderRadiusClassesFound.map((classFound) => {
            const havePrefix = classFound.match(PREFIX_REGEX);
            const borderRadiuslassNameSplitted = classFound.split('-');
            const rule = `border-radius: ${theme.spacing.borderRadius[borderRadiuslassNameSplitted[2]]};`;
            if (havePrefix?.length) {
                return `
          @media ${MediaQueries[havePrefix[0].replace(':', '').toUpperCase()]} {
            ${rule}
          }
        `;
            }
            return rule;
        });
    }
    return undefined;
};
const Container = styled__default["default"].div `
  ${generateBorderStyles}
  ${generateBoxShadowStyles}
  ${generateBorderRadiusStyles}
  &.grid {
    max-width: 1440px;
    width: 100%;

    ${({ $removeGridPadding }) => !$removeGridPadding &&
    styled.css `
        padding-left: ${({ theme }) => theme.tokens.flexbox.S};
        padding-right: ${({ theme }) => theme.tokens.flexbox.S};

        @media ${MediaQueries.SM} {
          padding-left: ${({ theme }) => theme.tokens.flexbox.L};
          padding-right: ${({ theme }) => theme.tokens.flexbox.L};
        }

        @media ${MediaQueries.MD} {
          padding-left: ${({ theme }) => theme.tokens.flexbox['2XL']};
          padding-right: ${({ theme }) => theme.tokens.flexbox['2XL']};
        }

        @media ${MediaQueries.LG} {
          padding-left: ${({ theme }) => theme.tokens.flexbox['3XL']};
          padding-right: ${({ theme }) => theme.tokens.flexbox['3XL']};
        }
      `}

    ${({ $removeGridGap }) => !$removeGridGap &&
    styled.css `
        column-gap: ${({ theme }) => theme.tokens.flexbox.XS};

        @media ${MediaQueries.SM} {
          column-gap: ${({ theme }) => theme.tokens.flexbox.S};
        }

        @media ${MediaQueries.MD} {
          column-gap: ${({ theme }) => theme.tokens.flexbox.M};
        }

        @media ${MediaQueries.LG} {
          column-gap: ${({ theme }) => theme.tokens.flexbox.M};
        }
      `}
  }

  ${({ $sizes }) => !!$sizes &&
    `
    flex: 0 1 ${$sizes.xs}%;

    @media ${MediaQueries.SM} {
      flex: 0 1 ${$sizes.sm}%;
    }

    @media ${MediaQueries.MD} {
      flex: 0 1 ${$sizes.md}%;
    }

    @media ${MediaQueries.LG} {
      flex: 0 1 ${$sizes.lg}%;
    }
  `}
`;

const Box = React.forwardRef(({ className, removeGridGap, removeGridPadding, sizes, children, onClick }, ref) => {
    const formatSizes = () => {
        const requiredKeys = ['xs', 'sm', 'md', 'lg'];
        if (sizes) {
            const finalSizes = sizes;
            const checkAllKeys = requiredKeys.every((key) => key in finalSizes && finalSizes[key]);
            return checkAllKeys ? sizes : undefined;
        }
        return undefined;
    };
    return (jsxRuntime.jsx(Container, { ref: ref, className: Array.isArray(className) ? className.join(' ').trim() : className, "$removeGridPadding": removeGridPadding, "$removeGridGap": removeGridGap, "$sizes": formatSizes(), onClick: onClick, children: children }));
});
Box.displayName = 'BoxContainer';

const CenterContainer = styled__default["default"].div `
  display: flex;
  max-width: 100vw;
  width: 100%;

  & > div {
    width: inherit;
    max-width: 1440px;
    margin: 0 auto;
  }
`;

var Center = React.forwardRef((props, ref) => {
    return (jsxRuntime.jsx(CenterContainer, { ref: ref, className: props.className, children: jsxRuntime.jsx("div", { children: props.children }) }));
});

const calculateChildrens = ($size) => {
    const childrens = Math.round(100 / parseInt($size.replace('%', '')));
    return childrens === 1 ? childrens + 1 : childrens;
};
const getDisplay = ($size) => styled.css `
  display: ${$size === '0%' ? 'none' : 'block'};
  ${$size === '0%' ? '.flex { display: none; }' : '.flex { display: flex; }'};
`;
const ColumnContainer = styled__default["default"].div `
  --childrens-xs: ${({ $size }) => calculateChildrens($size.xs)};
  --childrens-sm: ${({ $size }) => calculateChildrens($size.sm)};
  --childrens-md: ${({ $size }) => calculateChildrens($size.md)};
  --childrens-lg: ${({ $size }) => calculateChildrens($size.lg)};

  width: ${({ $size, theme }) => $size.xs !== '100%'
    ? `calc(${$size.xs} - (${theme.tokens.flexbox.XS} * (var(--childrens-xs) - 1)) / var(--childrens-xs))`
    : '100%'};

  ${({ $size }) => getDisplay($size.xs)};

  @media ${MediaQueries.SM} {
    width: ${({ $size, theme }) => $size.sm !== '100%'
    ? `calc(${$size.sm} - (${theme.tokens.flexbox.S} * (var(--childrens-sm) - 1)) / var(--childrens-sm))`
    : '100%'};

    ${({ $size }) => getDisplay($size.sm)};
  }

  @media ${MediaQueries.MD} {
    width: ${({ $size, theme }) => $size.md !== '100%'
    ? `calc(${$size.md} - (${theme.tokens.flexbox.M} * (var(--childrens-md) - 1)) / var(--childrens-md))`
    : '100%'};

    ${({ $size }) => getDisplay($size.md)};
  }

  @media ${MediaQueries.LG} {
    width: ${({ $size, theme }) => $size.lg !== '100%'
    ? `calc(${$size.lg} - (${theme.tokens.flexbox.M} * (var(--childrens-lg) - 1)) / var(--childrens-lg))`
    : '100%'};

    ${({ $size }) => getDisplay($size.lg)};
  }
`;

const Column = React.forwardRef(({ children, size, className, ...rest }, ref) => {
    return (jsxRuntime.jsx(ColumnContainer, { ...rest, ref: ref, className: className, "$size": size, children: children }));
});

const RowContainer = styled__default["default"].div `
  display: flex;
  flex-wrap: wrap;
  max-width: 1440px;
  gap: ${({ theme }) => theme.tokens.flexbox.XS};

  @media ${MediaQueries.SM} {
    gap: ${({ theme }) => theme.tokens.flexbox.S};
  }

  @media ${MediaQueries.MD} {
    gap: ${({ theme }) => theme.tokens.flexbox.M};
  }

  @media ${MediaQueries.LG} {
    gap: ${({ theme }) => theme.tokens.flexbox.M};
  }

  ${({ $center }) => $center &&
    styled.css `
      margin: 0 auto;
    `}

  ${({ $fullHeight }) => $fullHeight &&
    styled.css `
      height: 100%;
    `}

  ${({ $fullWidth }) => !$fullWidth &&
    styled.css `
      padding-left: ${({ theme }) => theme.tokens.flexbox.S};
      padding-right: ${({ theme }) => theme.tokens.flexbox.S};

      @media ${MediaQueries.SM} {
        padding-left: ${({ theme }) => theme.tokens.flexbox.L};
        padding-right: ${({ theme }) => theme.tokens.flexbox.L};
      }

      @media ${MediaQueries.MD} {
        padding-left: ${({ theme }) => theme.tokens.flexbox['2XL']};
        padding-right: ${({ theme }) => theme.tokens.flexbox['2XL']};
      }

      @media ${MediaQueries.LG} {
        padding-left: ${({ theme }) => theme.tokens.flexbox['3XL']};
        padding-right: ${({ theme }) => theme.tokens.flexbox['3XL']};
      }
    `}
`;

const Row = React.forwardRef(({ className, children, fullWidth = false, fullHeight = false, center = false, ...rest }, ref) => {
    return (jsxRuntime.jsx(RowContainer, { ...rest, ref: ref, "$fullWidth": fullWidth, "$fullHeight": fullHeight, "$center": center, className: className, children: children }));
});

/* eslint-disable @typescript-eslint/no-explicit-any */
const cssTypography = ({ theme, $align, $type, $variant, }) => styled.css `
  text-align: ${$align ? $align : 'left'};
  font-weight: ${theme.typography.mobile[$type][$variant].fontWeight};
  line-height: ${theme.typography.mobile[$type][$variant].lineHeight};
  font-size: ${theme.typography.mobile[$type][$variant].fontSize};
  letter-spacing: ${theme.typography.mobile[$type][$variant].letterSpacing};

  ${`@media only screen and (min-width: ${ScreenSizes.XS.max}px)`} {
    font-weight: ${theme.typography.desktop[$type][$variant].fontWeight};
    line-height: ${theme.typography.desktop[$type][$variant].lineHeight};
    font-size: ${theme.typography.desktop[$type][$variant].fontSize};
    letter-spacing: ${theme.typography.desktop[$type][$variant].letterSpacing};
  }

  a {
    font-weight: 700;
    color: inherit;

    &:visited {
      color: inherit;
    }
  }
`;
const TyporaphyHeading = styled__default["default"].h1 `
  ${cssTypography}
`;
const TyporaphyLabel = styled__default["default"].label `
  ${cssTypography}
`;
const TyporaphyParagraph = styled__default["default"].p `
  ${cssTypography}
`;
const TyporaphySpan = styled__default["default"].span `
  ${cssTypography}
`;

const elements = {
    headings: {
        h0: 'h1',
        h1: 'h1',
        h1Light: 'h1',
        h2: 'h2',
        h3: 'h3',
        h4: 'h4',
    },
    body: 'p',
    label: 'label',
    input: 'label',
    button: 'label',
};
const allowedProps = {
    blank: {
        target: '_blank',
    },
    download: {
        download: true,
    },
};
const Typography = React.forwardRef(({ className, children, type = 'body', variant = 'primary', text, align, ...props }, ref) => {
    const value = children || text;
    let linksMatched = [];
    let formatForReplace = '';
    let currentLink = 0;
    let links = [];
    if (value && typeof value === 'string') {
        const regex = new RegExp(/(\[[A-Za-zÀ-ÖØ-öø-ÿ ~+.!#%^&+@:/()*$_?=`|/\\-]+\](\([\w0-9+.!#%^&+@:/*$_?=`|/\\-]+\))(\((blank|download)\))?)+/g);
        linksMatched = value?.match(regex) || [];
        formatForReplace = value?.replace(regex, '${replace}');
        links = linksMatched?.map((matched, index) => {
            const text = matched.match(/\[[A-zÀ-ÖØ-öø-ÿ +.!#%^&+@:/()*$_?=`|/\\-]+\]/g)?.[0] || '';
            const formattedText = text.slice(1, -1);
            const link = matched.replace(text, '');
            const linkFormatted = link.replace(text, '').substring(link.indexOf('(') + 1, link.indexOf(')'));
            const linkProps = link.substring(link.lastIndexOf('(') + 1, link.lastIndexOf(')'));
            return (jsxRuntime.jsx("a", { href: linkFormatted, rel: "noopener", ...allowedProps[linkProps], ...uiUtils.applyDataAttributes(props, `link-${index + 1}`), children: formattedText }));
        });
    }
    const renderTypography = (localChildren) => {
        const elSelected = elements[type][variant] ||
            elements[type] ||
            'span';
        switch (elSelected) {
            case 'p':
                return (jsxRuntime.jsx(TyporaphyParagraph, { className: className, "$type": type, "$variant": variant, "$align": align, ref: ref, ...props, ...uiUtils.applyDataAttributes(props), children: localChildren }));
            case 'label':
                return (jsxRuntime.jsx(TyporaphyLabel, { className: className, "$type": type, "$variant": variant, "$align": align, ref: ref, ...props, ...uiUtils.applyDataAttributes(props), children: localChildren }));
            case 'span':
                return (jsxRuntime.jsx(TyporaphySpan, { className: className, "$type": type, "$variant": variant, "$align": align, ref: ref, ...props, ...uiUtils.applyDataAttributes(props), children: localChildren }));
            default:
                return (jsxRuntime.jsx(TyporaphyHeading, { as: elSelected, className: className, "$type": type, "$variant": variant, "$align": align, ref: ref, ...props, ...uiUtils.applyDataAttributes(props), children: localChildren }));
        }
    };
    return links && links?.length
        ? renderTypography(formatForReplace?.split(/(\${replace})/).map((word, index) => {
            return (jsxRuntime.jsx(React__default["default"].Fragment, { children: word === '${replace}' ? links && links[currentLink++] : word }, index));
        }))
        : renderTypography(value);
});

/* eslint-disable @typescript-eslint/no-explicit-any */
class Color {
    r;
    g;
    b;
    constructor(r, g, b) {
        this.r = r;
        this.g = g;
        this.b = b;
        this.set(r, g, b);
    }
    toString() {
        return `rgb(${Math.round(this.r)}, ${Math.round(this.g)}, ${Math.round(this.b)})`;
    }
    set(r, g, b) {
        this.r = this.clamp(r);
        this.g = this.clamp(g);
        this.b = this.clamp(b);
    }
    hueRotate(angle = 0) {
        angle = (angle / 180) * Math.PI;
        const sin = Math.sin(angle);
        const cos = Math.cos(angle);
        this.multiply([
            0.213 + cos * 0.787 - sin * 0.213,
            0.715 - cos * 0.715 - sin * 0.715,
            0.072 - cos * 0.072 + sin * 0.928,
            0.213 - cos * 0.213 + sin * 0.143,
            0.715 + cos * 0.285 + sin * 0.14,
            0.072 - cos * 0.072 - sin * 0.283,
            0.213 - cos * 0.213 - sin * 0.787,
            0.715 - cos * 0.715 + sin * 0.715,
            0.072 + cos * 0.928 + sin * 0.072,
        ]);
    }
    grayscale(value = 1) {
        this.multiply([
            0.2126 + 0.7874 * (1 - value),
            0.7152 - 0.7152 * (1 - value),
            0.0722 - 0.0722 * (1 - value),
            0.2126 - 0.2126 * (1 - value),
            0.7152 + 0.2848 * (1 - value),
            0.0722 - 0.0722 * (1 - value),
            0.2126 - 0.2126 * (1 - value),
            0.7152 - 0.7152 * (1 - value),
            0.0722 + 0.9278 * (1 - value),
        ]);
    }
    sepia(value = 1) {
        this.multiply([
            0.393 + 0.607 * (1 - value),
            0.769 - 0.769 * (1 - value),
            0.189 - 0.189 * (1 - value),
            0.349 - 0.349 * (1 - value),
            0.686 + 0.314 * (1 - value),
            0.168 - 0.168 * (1 - value),
            0.272 - 0.272 * (1 - value),
            0.534 - 0.534 * (1 - value),
            0.131 + 0.869 * (1 - value),
        ]);
    }
    saturate(value = 1) {
        this.multiply([
            0.213 + 0.787 * value,
            0.715 - 0.715 * value,
            0.072 - 0.072 * value,
            0.213 - 0.213 * value,
            0.715 + 0.285 * value,
            0.072 - 0.072 * value,
            0.213 - 0.213 * value,
            0.715 - 0.715 * value,
            0.072 + 0.928 * value,
        ]);
    }
    multiply(matrix) {
        const newR = this.clamp(this.r * matrix[0] + this.g * matrix[1] + this.b * matrix[2]);
        const newG = this.clamp(this.r * matrix[3] + this.g * matrix[4] + this.b * matrix[5]);
        const newB = this.clamp(this.r * matrix[6] + this.g * matrix[7] + this.b * matrix[8]);
        this.r = newR;
        this.g = newG;
        this.b = newB;
    }
    brightness(value = 1) {
        this.linear(value);
    }
    contrast(value = 1) {
        this.linear(value, -(0.5 * value) + 0.5);
    }
    linear(slope = 1, intercept = 0) {
        this.r = this.clamp(this.r * slope + intercept * 255);
        this.g = this.clamp(this.g * slope + intercept * 255);
        this.b = this.clamp(this.b * slope + intercept * 255);
    }
    invert(value = 1) {
        this.r = this.clamp((value + (this.r / 255) * (1 - 2 * value)) * 255);
        this.g = this.clamp((value + (this.g / 255) * (1 - 2 * value)) * 255);
        this.b = this.clamp((value + (this.b / 255) * (1 - 2 * value)) * 255);
    }
    hsl() {
        // Code taken from https://stackoverflow.com/a/9493060/2688027, licensed under CC BY-SA.
        const r = this.r / 255;
        const g = this.g / 255;
        const b = this.b / 255;
        const max = Math.max(r, g, b);
        const min = Math.min(r, g, b);
        const l = (max + min) / 2;
        let h, s;
        if (max === min) {
            h = s = 0;
        }
        else {
            const d = max - min;
            s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
            switch (max) {
                case r:
                    h = (g - b) / d + (g < b ? 6 : 0);
                    break;
                case g:
                    h = (b - r) / d + 2;
                    break;
                case b:
                    h = (r - g) / d + 4;
                    break;
            }
            h = h ? h / 6 : 0;
        }
        return {
            h: h * 100,
            s: s * 100,
            l: l * 100,
        };
    }
    clamp(value) {
        if (value > 255) {
            value = 255;
        }
        else if (value < 0) {
            value = 0;
        }
        return value;
    }
}
class Solver {
    target;
    targetHSL;
    reusedColor;
    constructor(target) {
        this.target = target;
        this.targetHSL = target.hsl();
        this.reusedColor = new Color(0, 0, 0);
    }
    solve() {
        const result = this.solveNarrow(this.solveWide());
        return {
            values: result.values,
            loss: result.loss,
            filter: this.css(result.values),
        };
    }
    solveWide() {
        const A = 5;
        const c = 15;
        const a = [60, 180, 18000, 600, 1.2, 1.2];
        let best = { loss: Infinity };
        for (let i = 0; best.loss > 25 && i < 3; i++) {
            const initial = [50, 20, 3750, 50, 100, 100];
            const result = this.spsa(A, a, c, initial, 1000);
            if (result.loss < best.loss) {
                best = result;
            }
        }
        return best;
    }
    solveNarrow(wide) {
        const A = wide.loss;
        const c = 2;
        const A1 = A + 1;
        const a = [0.25 * A1, 0.25 * A1, A1, 0.25 * A1, 0.2 * A1, 0.2 * A1];
        return this.spsa(A, a, c, wide.values, 500);
    }
    spsa(A, a, c, values, iters) {
        const alpha = 1;
        const gamma = 0.16666666666666666;
        let best = null;
        let bestLoss = Infinity;
        const deltas = new Array(6);
        const highArgs = new Array(6);
        const lowArgs = new Array(6);
        for (let k = 0; k < iters; k++) {
            const ck = c / Math.pow(k + 1, gamma);
            for (let i = 0; i < 6; i++) {
                deltas[i] = Math.random() > 0.5 ? 1 : -1;
                highArgs[i] = values[i] + ck * deltas[i];
                lowArgs[i] = values[i] - ck * deltas[i];
            }
            const lossDiff = this.loss(highArgs) - this.loss(lowArgs);
            for (let i = 0; i < 6; i++) {
                const g = (lossDiff / (2 * ck)) * deltas[i];
                const ak = a[i] / Math.pow(A + k + 1, alpha);
                values[i] = fix(values[i] - ak * g, i);
            }
            const loss = this.loss(values);
            if (loss < bestLoss) {
                best = values.slice(0);
                bestLoss = loss;
            }
        }
        return { values: best, loss: bestLoss };
        function fix(value, idx) {
            let max = 100;
            if (idx === 2 /* saturate */) {
                max = 7500;
            }
            else if (idx === 4 /* brightness */ || idx === 5 /* contrast */) {
                max = 200;
            }
            if (idx === 3 /* hue-rotate */) {
                if (value > max) {
                    value %= max;
                }
                else if (value < 0) {
                    value = max + (value % max);
                }
            }
            else if (value < 0) {
                value = 0;
            }
            else if (value > max) {
                value = max;
            }
            return value;
        }
    }
    loss(filters) {
        // Argument is array of percentages.
        const color = this.reusedColor;
        color.set(23, 15, 79); // Bolttech Navy
        color.invert(filters[0] / 100);
        color.sepia(filters[1] / 100);
        color.saturate(filters[2] / 100);
        color.hueRotate(filters[3] * 3.6);
        color.brightness(filters[4] / 100);
        color.contrast(filters[5] / 100);
        const colorHSL = color.hsl();
        return (Math.abs(color.r - this.target.r) +
            Math.abs(color.g - this.target.g) +
            Math.abs(color.b - this.target.b) +
            Math.abs(colorHSL.h - this.targetHSL.h) +
            Math.abs(colorHSL.s - this.targetHSL.s) +
            Math.abs(colorHSL.l - this.targetHSL.l));
    }
    css(filters) {
        function fmt(idx, multiplier = 1) {
            return Math.round(filters[idx] * multiplier);
        }
        return `filter: invert(${fmt(0)}%) sepia(${fmt(1)}%) saturate(${fmt(2)}%) hue-rotate(${fmt(3, 3.6)}deg) brightness(${fmt(4)}%) contrast(${fmt(5)}%);`;
    }
}
function hexToRgb(hex) {
    // Expand shorthand form (e.g. "03F") to full form (e.g. "0033FF")
    const shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
    hex = hex.replace(shorthandRegex, (r, g, b) => {
        return r + r + g + g + b + b;
    });
    const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? [parseInt(result[1], 16), parseInt(result[2], 16), parseInt(result[3], 16)] : null;
}
const getFilteredColor = (colorHash) => {
    const rgb = hexToRgb(colorHash);
    if (rgb) {
        const color = new Color(rgb[0], rgb[1], rgb[2]);
        const solver = new Solver(color);
        const result = solver.solve();
        return result.filter;
    }
    return '';
};

/* eslint-disable @typescript-eslint/no-explicit-any */
/**
 * This function is responsible to create class auxiliary for text colors and backgrounds
 * it uses a recursion to get to all possible properties of the theme and extract
 * all the colors used there and generate classes based on the properties names.
 * @param {object | string} currentObject
 * @param {string} property
 */
const getColorProperties = (currentObject, property = '') => {
    if (typeof currentObject === 'string') {
        return `
    .text${property} {
      color: ${currentObject};
    }
    .bg${property} {
      background-color: ${currentObject};
    }
    .fill${property} {
      ${getFilteredColor(currentObject)}
    }
  `;
    }
    else {
        return Object.keys(currentObject)
            .map((currentObjectKey) => {
            return getColorProperties(currentObject[currentObjectKey], `${property}-${currentObjectKey}`);
        })
            .join('');
    }
};
/**
 * This function is responsible to create class auxiliary for text colours and backgrounds
 * and backgrounds using the theme itself.
 * @param {ThemeType} theme
 */
const createColorClasses = (theme) => {
    return getColorProperties(theme.colors);
};
/**
 * This function is responsible to create class auxiliary for spacings
 * @param {object | string} currentObject
 * @param {string} property
 */
const getSpacingProperties = (spacingObject, property = '', propertyLetter = '', prefix = '') => {
    const escapedPrefix = typeof CSS !== 'undefined' ? CSS.escape(prefix) : prefix;
    return Object.keys(spacingObject)
        .map((currentObjectKey) => {
        const lowerCaseObjectKey = currentObjectKey.toLowerCase();
        const value = spacingObject[currentObjectKey];
        return `
      .${escapedPrefix}${propertyLetter}t-${lowerCaseObjectKey} {
        ${property}-top: ${value};
      }

      .${escapedPrefix}${propertyLetter}b-${lowerCaseObjectKey} {
        ${property}-bottom: ${value};
      }

      .${escapedPrefix}${propertyLetter}l-${lowerCaseObjectKey} {
        ${property}-left: ${value};
      }

      .${escapedPrefix}${propertyLetter}r-${lowerCaseObjectKey} {
        ${property}-right: ${value};
      }

      .${escapedPrefix}${propertyLetter}-${lowerCaseObjectKey} {
        ${property}: ${value};
      }

      .${escapedPrefix}${propertyLetter}h-${lowerCaseObjectKey} {
        ${property}-left: ${value};
        ${property}-right: ${value};
      }

      .${escapedPrefix}${propertyLetter}v-${lowerCaseObjectKey} {
        ${property}-top: ${value};
        ${property}-bottom: ${value};
      }
    `;
    })
        .join('');
};
/**
 * This function is responsible to create class auxiliary for spacings using the theme itself.
 * @param {ThemeType} theme
 */
const createPaddingClasses = (theme, prefix) => {
    return getSpacingProperties(theme.tokens?.['padding'], 'padding', 'p', prefix);
};
/**
 * This function is responsible to create class auxiliary for spacings using the theme itself.
 * @param {ThemeType} theme
 */
const createMarginClasses = (theme, prefix) => {
    return getSpacingProperties(theme.tokens?.['flexbox'], 'margin', 'm', prefix);
};
const createPositioningClasses = (prefix = '') => {
    const escapedPrefix = typeof CSS !== 'undefined' ? CSS.escape(prefix) : prefix;
    return `
    .${escapedPrefix}flex-row { flex-direction: row; }
    .${escapedPrefix}flex-row-reverse { flex-direction: row-reverse; }
    .${escapedPrefix}flex-column { flex-direction: column; }
    .${escapedPrefix}flex-column-reverse { flex-direction: column-reverse; }
    .${escapedPrefix}flex { display: flex; }
    .${escapedPrefix}flex-grow { flex: 1; }
    .${escapedPrefix}flex-shrink-1 { flex-shrink: 1; }
    .${escapedPrefix}flex-shrink-0 { flex-shrink: 0; }
    .${escapedPrefix}flex-grow-1 { flex-grow: 1; }
    .${escapedPrefix}flex-grow-0 { flex-grow: 0; }
    .${escapedPrefix}flex-basis-0 { flex-basis: 0; }
    .${escapedPrefix}flex-basis-1 { flex-basis: auto; }
    .${escapedPrefix}flex-wrap { flex-wrap: wrap; }
    .${escapedPrefix}flex-no-wrap { flex-wrap: nowrap; }
    .${escapedPrefix}align-start { align-items: flex-start; }
    .${escapedPrefix}align-center { align-items: center; }
    .${escapedPrefix}align-right { align-items: flex-end; }
    .${escapedPrefix}align-self-stretch { align-self: stretch; }
    .${escapedPrefix}align-self-center { align-self: center; }
    .${escapedPrefix}align-self-start { align-self: start; }
    .${escapedPrefix}align-self-end { align-self: end; }
    .${escapedPrefix}justify-left { justify-content: flex-start; }
    .${escapedPrefix}justify-center { justify-content: center; }
    .${escapedPrefix}justify-right { justify-content: flex-end; }
    .${escapedPrefix}justify-between { justify-content: space-between; }
    .${escapedPrefix}justify-evenly { justify-content: space-evenly; }
    .${escapedPrefix}justify-around { justify-content: space-around; }
    .${escapedPrefix}max-content { width: max-content; }
    .${escapedPrefix}min-content { width: min-content; }
    .${escapedPrefix}gap-0 { gap: 0; }
    .${escapedPrefix}mh-auto { margin-left: auto; margin-right: auto; }
  `;
};
const createTypographyClasses = (theme) => {
    const generateClass = (typographyVariant, variantName) => {
        return `${Object.entries(typographyVariant).map((variantEntry) => {
            const bodyKeyEntry = variantEntry[0];
            const classNamePrefix = `.ty-${variantName}-${bodyKeyEntry}`.toLowerCase();
            const bodyStylesEntry = variantEntry[1];
            return `
      ${classNamePrefix} {
        font-weight: ${bodyStylesEntry.fontWeight};
        line-height: ${bodyStylesEntry.lineHeight};
        font-size: ${bodyStylesEntry.fontSize};
        letter-spacing: ${bodyStylesEntry.letterSpacing};
      }
    `;
        })}`;
    };
    return `
    ${generateClass(theme.typography.mobile.headings, 'headings')}
    ${generateClass(theme.typography.mobile.body, 'body')}
    ${generateClass(theme.typography.mobile.label, 'label')}

    @media only screen and (min-width: ${ScreenSizes.XS.max}px) {
      ${generateClass(theme.typography.desktop.headings, 'headings')}
      ${generateClass(theme.typography.desktop.body, 'body')}
      ${generateClass(theme.typography.desktop.label, 'label')}
    }
  `;
};
const createGapClasses = (theme, prefix = '') => {
    const gapClassName = `.${prefix}`;
    return Object.entries(theme.tokens?.flexbox).map((spacingEntry) => {
        const flexboxEntry = spacingEntry[0];
        const flexEntry = spacingEntry[1];
        return `
      ${`${gapClassName}gap-${flexboxEntry}`.toLowerCase()} {
        gap: ${flexEntry};
      }
      ${`${gapClassName}rg-${flexboxEntry}`.toLowerCase()} {
        row-gap: ${flexEntry};
      }
      ${`${gapClassName}cg-${flexboxEntry}`.toLowerCase()} {
        column-gap: ${flexEntry};
      }
    `;
    });
};
const createUtilitaryClasses = (theme) => {
    return `
  :root {
    ${createTypographyClasses(theme)}
    ${createColorClasses(theme)}
    ${createMarginClasses(theme)}
    ${createPaddingClasses(theme)}
    ${createPositioningClasses()}
    ${createGapClasses(theme)}

    @media (min-width: ${ScreenSizes.SM.min}px) {
      ${createPositioningClasses('sm:')}
      ${createMarginClasses(theme, 'sm:')}
      ${createPaddingClasses(theme, 'sm:')}
      ${createGapClasses(theme, 'sm:')}
    }

    @media (min-width: ${ScreenSizes.MD.min}px) {
      ${createPositioningClasses('md:')}
      ${createMarginClasses(theme, 'md:')}
      ${createPaddingClasses(theme, 'md:')}
      ${createGapClasses(theme, 'md:')}
    }

    @media (min-width: ${ScreenSizes.LG.min}px) {
      ${createPositioningClasses('lg:')}
      ${createMarginClasses(theme, 'lg:')}
      ${createPaddingClasses(theme, 'lg:')}
      ${createGapClasses(theme, 'lg:')}
    }
  }`;
};
const createUtilitaryClassNames = (theme) => {
    let classesProcessed = { color: [], spacing: [], positioning: [] };
    const classesToGenerate = [
        { key: 'color', content: `${createColorClasses(theme)}` },
        {
            key: 'spacing',
            content: `
        ${createMarginClasses(theme)}
        ${createPaddingClasses(theme)}
        ${createMarginClasses(theme, 'sm:')}
        ${createPaddingClasses(theme, 'sm:')}
        ${createMarginClasses(theme, 'md:')}
        ${createPaddingClasses(theme, 'md:')}
        ${createMarginClasses(theme, 'lg:')}
        ${createPaddingClasses(theme, 'lg:')}
      `,
        },
        {
            key: 'positioning',
            content: `
        ${createPositioningClasses()}
        ${createPositioningClasses('sm:')}
        ${createPositioningClasses('md:')}
        ${createPositioningClasses('lg:')}
      `,
        },
    ];
    classesToGenerate.forEach((classContent) => {
        const classesFound = classContent.content.match(/\.\S+/g);
        if (classesFound) {
            classesProcessed = {
                ...classesProcessed,
                [classContent.key]: [...classesFound].map((className) => className.replace(/\\|(\.)/gm, '')),
            };
        }
    });
    return classesProcessed;
};

/**
 * An array representing a range of sizes from 0 to 100.
 */
new Array(101).fill(0);

const UtilitaryClasses = styled.createGlobalStyle `
  ${styled.css `
    ${(props) => createUtilitaryClasses(props.theme)}
  `}
`;
const BolttechThemeProvider = ({ children, theme = defaultTheme.bolttechTheme }) => (jsxRuntime.jsxs(styled.ThemeProvider, { theme: theme, children: [jsxRuntime.jsx(UtilitaryClasses, { theme: theme }), children] }));

exports.BolttechThemeProvider = BolttechThemeProvider;
exports.Box = Box;
exports.Center = Center;
exports.Column = Column;
exports.MediaQueries = MediaQueries;
exports.Row = Row;
exports.ScreenSizes = ScreenSizes;
exports.Typography = Typography;
exports.createUtilitaryClassNames = createUtilitaryClassNames;
